<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="dolphinpart" version="15" translationDomain="dolphin">
 <MenuBar>
  <Menu name="edit"><text>&amp;Edit</text>
   <Action name="new_menu"/>
   <Separator/>
   <Action name="renamefile"/>
   <Action name="movetotrash" />
   <Action name="deletefile"/>
   <Action name="editMimeType"/>
   <Action name="properties"/>
   <Separator/>
   <Menu name="selection">
    <text context="@title:menu">Selection</text>
    <Action name="select_items_matching" />
    <Action name="unselect_items_matching" />
    <Separator/>
    <Action name="edit_select_all" />
    <Action name="unselect_all" />
    <Action name="invert_selection" />
   </Menu>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
   <Action name="sort" />
   <Action name="additional_info" />
   <Action name="show_preview" />
   <Action name="show_in_groups" />
   <Action name="show_hidden_files" />
   <Separator/>
   <Action name="view_properties" />
  </Menu>
  <Menu name="go"><text>&amp;Go</text>
    <Action name="go_applications"/>
    <Action name="go_network_folders"/>
    <Action name="go_settings"/>
    <Action name="go_media"/>
    <Action name="go_trash"/>
    <Action name="go_autostart"/>
  </Menu>
  <Menu name="tools"><text context="@title:menu">Tools</text>
    <Action name="open_terminal"/>
    <Action name="focus_terminal_panel"/>
    <Action name="find_file" />
    <Action name="show_filter_bar" />
    <Action name="compare_files" />
    <Action name="change_remote_encoding" />
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text context="@title:menu">Dolphin Toolbar</text>
  <Action name="icons" />
  <Action name="compact" />
  <Action name="details" />
</ToolBar>
<State name="has_selection" >
  <enable>
   <Action name="movetotrash" />
   <Action name="deletefile" />
  </enable>
 </State>
 <State name="has_no_selection" >
  <disable>
   <Action name="renamefile" />
   <Action name="movetotrash" />
   <Action name="deletefile" />
  </disable>
</State>
</gui>
